@@ -1042,7 +1042,7 @@ static gpasmVal do_direct(gpasmVal r,
     p= HEAD(TAIL(parms));
     if (p->tag == string) {
       if (strlen(p->value.string) < 255) {
-        direct_string = p->value.string;
+        direct_string = convert_escaped_char(p->value.string,'"');
       } else {
         gperror(GPE_UNKNOWN, "string must be less than 255 bytes long");
       }
@@ -2005,7 +2005,10 @@ static gpasmVal do_org(gpasmVal r,
       }	else {
         /* Default section name, this will be overwritten if a label is 
            present. */
-        snprintf(state.obj.new_sec_name, sizeof(state.obj.new_sec_name), ".org_%d", state.obj.org_num++);
+        snprintf(state.obj.new_sec_name,
+                 sizeof(state.obj.new_sec_name),
+                 ".org_%x",
+                 r);
         state.obj.new_sec_addr = r;
         state.obj.new_sec_flags = STYP_TEXT | STYP_ABS;
         state.lst.line.linetype = sec;
