@@ -126,63 +126,6 @@ gp_link_add_symbols(struct symbol_table *definition,
   return 0;
 }
 
-/* Determine if any relocation uses the symbol. */
-
-static gp_boolean
-_has_relocation(gp_object_type *object, gp_symbol_type *symbol)
-{
-  gp_section_type *section;
-  gp_reloc_type *relocation;
-
-  section = object->sections;
-  while (section != NULL) {
-    relocation = section->relocations;
-    while (relocation != NULL) {
-      if (relocation->symbol == symbol) {
-        return true;
-      }
-      relocation = relocation->next;
-    }
-    section = section->next;
-  }
-
-  return false;
-}
-
-/* Remove any weak symbols in the object. */
-
-void
-gp_cofflink_remove_weak(gp_object_type *object)
-{
-  gp_symbol_type *symbol;
-  gp_symbol_type *previous = NULL;
-
-  gp_debug("removing weak symbols from %s", object->filename);
-
-  /* Search the symbol table for extern symbols. */
-  symbol = object->symbols;
-  while (symbol != NULL) {
-    if ((symbol->class == C_EXT) && 
-        (symbol->section_number == N_UNDEF) &&
-        (!_has_relocation(object, symbol))) {
-      gp_debug("  removed weak symbol \"%s\"", symbol->name);
-      if (previous == NULL) {
-        /* removing first symbol in the list */
-        object->symbols = object->symbols->next;
-      } else {
-        previous->next = symbol->next;
-      }
-      object->num_symbols--;    
-    } else {
-      previous = symbol;
-    }
-
-    symbol = symbol->next;
-  }
-
-  return;
-}
-
 /* Combine all sections and symbols from all objects into one object file. */
 
 void
@@ -233,6 +176,63 @@ gp_cofflink_combine_objects(gp_object_type *object)
   return;
 }
 
+/* Cleanup the symbol table after combining objects. */
+
+void 
+gp_cofflink_clean_table(gp_object_type *object,
+                        struct symbol_table *symbols)
+{
+  gp_section_type    *section = object->sections;
+  gp_reloc_type      *relocation;
+  gp_symbol_type *symbol;
+  gp_coffsymbol_type *var;
+  struct symbol      *sym;
+  gp_symbol_type *previous = NULL;
+
+  gp_debug("cleaning symbol table");
+
+  /* point all relocations to the symbol definitions */
+  while (section != NULL) {
+    relocation = section->relocations;
+    while (relocation != NULL) {
+      symbol = relocation->symbol;
+      if ((symbol->class == C_EXT) && 
+          (symbol->section_number == N_UNDEF)) {
+        /* This is an external symbol defined elsewhere */
+        sym = get_symbol(symbols, symbol->name);
+        assert(sym != NULL);
+        var = get_symbol_annotation(sym);
+        assert(var != NULL);	  
+        relocation->symbol = var->symbol;
+      }
+      relocation = relocation->next;
+    }
+    section = section->next;
+  }
+
+  symbol = object->symbols;
+  while (symbol != NULL) {
+    if ((symbol->class == C_EXT) && 
+        (symbol->section_number == N_UNDEF)) {
+      /* This is an external symbol defined elsewhere */
+      if (previous == NULL) {
+        /* removing first symbol in the list */
+        object->symbols = object->symbols->next;
+      } else {
+        previous->next = symbol->next;
+      }
+      gp_debug("  removed symbol \"%s\"", symbol->name);
+      object->num_symbols--;
+    } else {
+      previous = symbol;
+    }
+
+    symbol = symbol->next;
+  }
+
+  return;
+}
+
 /* Update the line number offsets */
 
 static void
@@ -253,9 +253,7 @@ gp_cofflink_combine_overlay(gp_object_type *object, int remove_symbol)
 {
   gp_section_type *first = NULL;
   gp_section_type *second = NULL;
-  gp_section_type *list = NULL;
   gp_symbol_type  *symbol = NULL;
-  int count;
 
   first = object->sections;
   
@@ -284,17 +282,7 @@ gp_cofflink_combine_overlay(gp_object_type *object, int remove_symbol)
 
         /* Remove the section symbol */
         if (remove_symbol) {
-          symbol = object->symbols;
-          while (symbol != NULL) {
-            if (symbol->next == second->symbol) {
-              symbol->next = second->symbol->next;
-              /* FIXME : count = gp_coffgen_free_symbol(second->symbol); */
-              count = 1;
-              object->num_symbols -= (count + 1);
-              break;
-            }
-            symbol = symbol->next;
-          }
+          gp_coffgen_delsymbol(object, second->symbol);
         }
 
         /* Update the symbol table */
@@ -307,16 +295,7 @@ gp_cofflink_combine_overlay(gp_object_type *object, int remove_symbol)
         }
 
         /* Remove the second section*/
-        list = object->sections;
-        while(list != NULL) {
-          if (list->next == second) {
-            list->next = second->next;
-            break;
-          }
-          list = list->next;
-        }
-        object->num_sections--;
-        /* FIXME: gp_coffgen_free_section(second); */
+        gp_coffgen_delsection(object, second);
         
         /* Take another pass */
         gp_cofflink_combine_overlay(object, remove_symbol);
@@ -388,7 +367,6 @@ gp_cofflink_merge_sections(gp_object_type *object, int byte_addr)
 {
   gp_section_type *first;
   gp_section_type *second;
-  gp_section_type *list = NULL;
   gp_symbol_type  *symbol = NULL;
   gp_reloc_type   *relocation = NULL;
   unsigned int org;
@@ -490,21 +468,7 @@ gp_cofflink_merge_sections(gp_object_type *object, int byte_addr)
       }
 
       /* Remove the second section*/
-      list = object->sections;
-      while(list != NULL) {
-        if (list->next == second) {
-          list->next = second->next;
-          if (list->next == NULL) {
-            /* The last section in the list is being removed, so update
-               the tail. */ 
-            object->sections_tail = list;
-          }
-          break;
-        }
-        list = list->next;
-      }
-      object->num_sections--;
-      /* FIXME: gp_coffgen_free_section(second); */
+      gp_coffgen_delsection(object, second);
         
       /* Take another pass */
       gp_cofflink_merge_sections(object, byte_addr);
@@ -630,6 +594,30 @@ _read_table_data(gp_section_type *section, int org)
   return data;
 }
 
+/* create the symbol for the start address of the table */
+
+void
+gp_cofflink_make_cinit(gp_object_type *object)
+{
+  gp_symbol_type *symbol;
+
+  /* create the symbol for the start address of the table */
+  symbol = gp_coffgen_findsymbol(object, "_cinit");
+  if ((symbol != NULL) && (symbol->section_number > 0)) {
+    gp_error("_cinit symbol already exists");
+  } else {
+    symbol = gp_coffgen_addsymbol(object);
+    symbol->name           = strdup("_cinit");
+    symbol->value          = 0;
+    symbol->section_number = 1;
+    symbol->section        = NULL;
+    symbol->type           = T_NULL;
+    symbol->class          = C_EXT;
+  }
+
+  return;
+}
+
 /* create ROM data for initialized data sections */
 
 void
@@ -675,19 +663,10 @@ gp_cofflink_make_idata(gp_object_type *object)
       _write_table_data(new, 0, insn, count);
     }
 
-    /* create the symbol for the start address of the table */
+    /* update the section pointer in _cinit */
     symbol = gp_coffgen_findsymbol(object, "_cinit");
-    if ((symbol != NULL) && (symbol->section_number > 0)) {
-      gp_error("_cinit symbol already exists");
-    } else {
-      symbol = gp_coffgen_addsymbol(object);
-      symbol->name           = strdup("_cinit");
-      symbol->value          = 0;
-      symbol->section_number = 1;
-      symbol->section        = new;
-      symbol->type           = T_NULL;
-      symbol->class          = C_EXT;
-    }
+    assert(symbol != NULL);
+    symbol->section = new;
 
   }
 
@@ -1564,14 +1543,11 @@ gp_cofflink_patch_addr(enum proc_class class,
    stripped from the sections. */
 
 void 
-gp_cofflink_patch(gp_object_type *object,
-                  struct symbol_table *symbols)
+gp_cofflink_patch(gp_object_type *object)
 {
   gp_section_type    *section = object->sections;
   gp_reloc_type      *relocation;
   gp_symbol_type     *symbol;
-  gp_coffsymbol_type *var;
-  struct symbol      *sym;
   int num_pages;
   int num_banks;
   int bsr_boundary = 0;
@@ -1590,18 +1566,6 @@ gp_cofflink_patch(gp_object_type *object,
       relocation = section->relocations;
       while (relocation != NULL) {
         symbol = relocation->symbol;
-        if ((symbol->class == C_EXT) && 
-            (symbol->section_number == 0)) {
-          /* This is an external symbol defined elsewhere */
-          sym = get_symbol(symbols, symbol->name);
-          assert(sym != NULL);
-          var = get_symbol_annotation(sym);
-          assert(var != NULL);	  
-          symbol = var->symbol;
-          gp_debug("  lookup symbol %s (%#x)",
-                   symbol->name,
-                   symbol->value);
-        }
         gp_cofflink_patch_addr(object->class,
 	                       num_pages,
                                num_banks,
@@ -1631,38 +1595,6 @@ gp_cofflink_patch(gp_object_type *object,
   return;
 }
 
-/* Cleanup the symbol table after relocating and linking. */
-
-void 
-gp_cofflink_clean_table(gp_object_type *object)
-{
-  gp_symbol_type *previous = NULL;
-  gp_symbol_type *symbol = object->symbols;
-
-  gp_debug("cleaning symbol table");
-
-  while (symbol != NULL) {
-    if ((symbol->class == C_EXT) && 
-        (symbol->section_number == N_UNDEF)) {
-      /* This is an external symbol defined elsewhere */
-      if (previous == NULL) {
-        /* removing first symbol in the list */
-        object->symbols = object->symbols->next;
-      } else {
-        previous->next = symbol->next;
-      }
-      gp_debug("  removed symbol \"%s\"", symbol->name);
-      object->num_symbols--;
-    } else {
-      previous = symbol;
-    }
-
-    symbol = symbol->next;
-  }
-
-  return;
-}
-
 /* copy all executatable data to new memory */
 
 MemBlock * 
